/*
 * Access a filesystem and list a directory.
 *
 * Copyright (C) 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <dirent.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>

#include <fsclient/client.h>
#include <fsclient/client_fcntl.h>



static int flags = OPENDIR_DIRENT;

/* Show the directory entries. */

static void show_directory(file_descriptor_t *desc)
{
  struct dirent *dirent;
  struct stat *st;

  while ((dirent = client_readdir(desc)) != NULL)
  {
    fputs(dirent->d_type == DT_DIR ? "d" : "-", stdout);
    fputs(" ", stdout);

    if (flags & OPENDIR_STAT)
    {
      st = (struct stat *) ((char *) dirent + dirent->d_reclen);

      printf("%6o %5d %5d %6ld", st->st_mode, st->st_uid, st->st_gid, st->st_size);
      fputs(" ", stdout);
    }

    /* The name should be padded with at least one zero byte. */

    fputs(dirent->d_name, stdout);
    fputs("\n", stdout);
  }
}

int main(int argc, char *argv[])
{
  file_descriptor_t *desc1;

  if (argc < 2)
  {
    printf("Need a directory name.\n");
    return 1;
  }

  if ((argc > 2) && !strcmp(argv[2], "stat"))
    flags = OPENDIR_STAT;

  desc1 = client_open(argv[1], O_DIRECTORY | flags, 0);

  if (desc1 == NULL)
  {
    printf("Could not open directory.\n");
    return 1;
  }

  printf("Showing directory...\n");

  show_directory(desc1);

  printf("Closing directory.\n");
  client_close(desc1);

  printf("Closed.\n");
  return 0;
}
